<?php

namespace Drupal\webfinger\Controller;

use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Controller\ControllerBase;
use Drupal\webfinger\JsonRd;
use Drupal\webfinger\Event\WebfingerResponseEvent;
use Drupal\webfinger\WebfingerEvents;
use Drupal\webfinger\WebfingerParameters;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class WebfingerController.
 */
class WebfingerController extends ControllerBase {

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Constructs a \Drupal\aggregator\Controller\AggregatorController object.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger channel.
   */
  public function __construct(EventDispatcherInterface $event_dispatcher, LoggerInterface $logger) {
    $this->eventDispatcher = $event_dispatcher;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('event_dispatcher'),
      $container->get('logger.factory')->get('webfinger')
    );
  }

  /**
   * Handle request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Information about the current HTTP request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function handleRequest(Request $request): CacheableJsonResponse {
    $response_cacheability = new CacheableMetadata();
    $params = (new WebfingerParameters())
      ->setRequest($request)
      ->setResponseCacheability($response_cacheability)
      ->setLogger($this->logger)
      ->getParams();

    $json_rd = new JsonRd();
    $event = new WebfingerResponseEvent($json_rd, $request, $params, $response_cacheability);
    $this->eventDispatcher->dispatch(WebfingerEvents::WEBFINGER_BUILD_RESPONSE, $event);

    $response = new CacheableJsonResponse($event->getJsonRd()->toArray());
    $response->headers->set('Content-Type', 'application/jrd+json; charset=utf-8');
    $response->headers->set('Access-Control-Allow-Origin', '*');
    $response->addCacheableDependency($response_cacheability);
    // Triggering \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
    // returns an HTML response, while we want JSON.
    if (empty($event->getJsonRd()->getLinks())) {
      // If no links were returned, set a not found status code.
      $response->setStatusCode(404);
      $response->headers->set('Status', '404 Not Found');
    }

    return $response;
  }

}
